﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class EnemyController : MonoBehaviour {

	public float jumpHeight, velocity, topVelocity, ghostVelocity, health;
	public int life = 6, life2 = 2, lastHitter = 0;
	public bool ghostMode = false, completed = false;

	public List<GameObject> plataforms, fernets;
	SpriteRenderer spRenderer;

	public GameManager manager;

	GameObject targetPlataform = null, childPlataform = null, fernet = null;
	bool canJump = true, possible = true, clean = true, firstMove = true, forceMode = false;
	Rigidbody2D rb;
	int frames = 0, frames2 = 0, dir = 0;
	Vector3 lastPos;
	Animator animator;

	public bool Damage(int damage, int hitter) {
		if (health <= 0)
			return false;
		health -= damage;

		lastHitter = hitter;

		if (health <= 0) {
			animator.SetTrigger("muerto");
			return true;
		}
		
		return false;
	}

	IEnumerator drink() {
		yield return new WaitForSeconds(0.5f);

		manager.drinkFernet();

		if (health > 0)
			StartCoroutine(drink());
		else {
			spRenderer.enabled = false;
			gameObject.SetActive(false);
			Destroy(gameObject);
		}
	}

	public void throwBack(Vector3 origin, float intensity) {
		rb.AddForce((transform.position - origin) * -intensity);
	}

	void Start () {
		spRenderer = GetComponentInChildren<SpriteRenderer>();
		animator = GetComponent<Animator>();
		if (fernets != null)
			fernet = fernets[Random.Range(0, fernets.Count)];
		rb = GetComponent<Rigidbody2D> ();
		lastPos = transform.position;

		if (targetPlataform != null)
			calcTargetPlataform();

		StartCoroutine(ghostBusters());
		StartCoroutine(restoreJump());
	}

	public void chooseFernet() {
		fernet = fernets[Random.Range(0, fernets.Count)];
	}

	IEnumerator ghostBusters() {
		yield return new WaitForSeconds(life);
		forceMode = true;
		//StartCoroutine(ghostBusters2());
	}

	IEnumerator ghostBusters2() {
		yield return new WaitForSeconds(life2);
		GhostMode();
	}

	public void Destroy() {
		Destroy(gameObject);
	}

	void Update () {

		if (Input.GetKeyDown(KeyCode.Y))
			Debug.Log(canJump);

		//Plataforms
		if (!possible)
			GhostMode();

		if (completed) {
			return;
		}

		if (lastPos.y > transform.position.y) {
			if (dir != -1) {
				frames2 = 0;
				dir = -1;
			}

			frames2++;
			if (frames > 5)
				falling(dir);
		} else if (lastPos.y < transform.position.y) {
			if (dir != 1) {
				frames2 = 0;
				dir = 1;
			}

			frames2++;
			if (frames > 5)
				falling(dir);
		} else if (dir != 0) {
			frames2 = 0;
			dir = 0;
		}
			
		if (!firstMove) {
			if (lastPos == transform.position)
				frames++;
			else
				frames = 0;

			if (frames > 10)
				GhostMode();
		}

		lastPos = transform.position;
		if (ghostMode && !completed)
			moveTo(new Vector2(fernet.transform.position.x, fernet.transform.position.y) );
		else if (forceMode && !completed) {
			moveHorizontallyTo(new Vector2(transform.position.x > 0 ? -10 : 10, 0), ForceMode2D.Force, 1.5f);
		} else {
		if (firstMove) {
			RaycastHit2D hit = Physics2D.Raycast (transform.position, Vector2.zero, 1f);
				if (hit.transform != null)
				moveHorizontallyTo(new Vector2(-transform.position.x, -Mathf.Abs(transform.position.y)), ForceMode2D.Force, 1.5f);
			
			if (transform.position.y < lastPos.y - 1) {
				childPlataform = null;
				firstMove = false;
			}
		} else if (fernet.transform.position.y < transform.position.y + jumpHeight && targetPlataform == null) {
			moveHorizontallyTo (new Vector2 (fernet.transform.position.x, fernet.transform.position.y), ForceMode2D.Force, 1.5f);
		} else {
			if (clean && targetPlataform == null)
				calcTargetPlataform();

			if (transform.position.y > targetPlataform.transform.position.y) {
					toCenter();
			} else if (childPlataform == null && targetPlataform.transform.position.y <= transform.position.y + jumpHeight) {
				ToPlataform (targetPlataform);
			} else if (targetPlataform.transform.position.y < transform.position.y) {
				//Fernet
				moveHorizontallyTo (new Vector2 (fernet.transform.position.x, fernet.transform.position.y), ForceMode2D.Force, 1.5f);
			} else if (targetPlataform.transform.position.y > transform.position.y + jumpHeight) {
					int reps = 0;
					while ((childPlataform == null || childPlataform.transform.position.y > transform.position.y + jumpHeight) && reps < 10) {
					GameObject refPlataform = childPlataform == null ? targetPlataform : childPlataform;

					List<GameObject> possibilities = new List<GameObject> ();

					foreach(GameObject plataform in plataforms) {
						if (IsSuitablePlataform(plataform, refPlataform)) {
							possibilities.Add(plataform);
						}
					}

						if (possibilities.Count == 0) {
						if (childPlataform == null)
							possible = false;
						else {
							childPlataform = null;
							return;
						}
					} else 
							childPlataform = possibilities[Random.Range(0, possibilities.Count)];
						reps++;
				}
				if (childPlataform != null) {
					if (transform.position.y > childPlataform.transform.position.y) {
						toCenter();
					} else
						ToPlataform (childPlataform);
				}
			}
		}
		}
	}

	void GhostMode() {
		ghostMode = true;
		animator.SetBool("ghost", true);
		rb.gravityScale = 0;
		GetComponent<Collider2D>().isTrigger = true;
	}

	void Stick() {
		animator.SetTrigger("chupando");
		spRenderer.flipY = transform.position.x > 0;

		animator.SetBool("seMueve", false);
		animator.SetBool("saltando 0", false);
		animator.SetBool("cayendo", false);
	}

	void toCenter() {
		GameObject plataform = childPlataform == null ? targetPlataform : childPlataform;
		if (plataform != null)
		if (moveHorizontallyTo(new Vector2(plataform.transform.position.x, plataform.transform.position.y), ForceMode2D.Force, 1f) > 1f)
		if (childPlataform == null)
			targetPlataform = null;
		else childPlataform = null;
	}

	bool IsSuitablePlataform(GameObject plataform, GameObject compared) {
		if (plataform.transform.position.y >= compared.transform.position.y - jumpHeight) {
			float capX = 0;
			if (plataform.transform.position.y > compared.transform.position.y)
				capX = (plataform.transform.position.y - compared.transform.position.y) * 2.5f;
			else
				capX = jumpHeight - 1 + (compared.transform.position.y - plataform.transform.position.y);
			if (Mathf.Max (compared.transform.position.x, plataform.transform.position.x) - Mathf.Min (compared.transform.position.x, plataform.transform.position.x) <= capX)
				return true;
		}
		return false;
	}

	public void calcTargetPlataform() {
		targetPlataform = null;
		List<GameObject> possibilities = new List<GameObject> ();

		foreach (GameObject plataform in plataforms) {
			if (IsSuitablePlataform(plataform, fernet))
					possibilities.Add (plataform);
		}

		if (possibilities.Count == 0)
			possible = false;
		else
			targetPlataform = possibilities[Random.Range(0, possibilities.Count)];

		if (targetPlataform == null)
			clean = false;
	}

	void ToPlataform (GameObject plataform) {
		if (Random.Range(1, 3) > 2)
			moveHorizontallyTo(new Vector2(fernet.transform.position.x, fernet.transform.position.y), ForceMode2D.Force, 2f);
		else
			moveHorizontallyTo(new Vector2(plataform.transform.position.x, plataform.transform.position.y), ForceMode2D.Force, 0.5f);
	}

	void jump() {
		if (!ghostMode && !completed)
			rb.AddForce (new Vector2 (0, jumpHeight * 10f), ForceMode2D.Impulse);
	}

	IEnumerator restoreJump() {
		yield return new WaitForSeconds(Random.Range(1.2f, 1.8f));
		jump();
		if (!ghostMode && !completed)
			StartCoroutine(restoreJump());
	}

	void falling(int direction) {
		if (direction > 0) {
			animator.SetBool("saltando 0", true);
			animator.SetBool("cayendo", false);
		} else {
			animator.SetBool("saltando 0", false);
			animator.SetBool("cayendo", true);
		}
	}

	void OnCollisionEnter2D (Collision2D coll) {
		if (coll.gameObject.CompareTag("Fernet")) {
			transform.parent = fernet.transform;
			rb.velocity = Vector2.zero;
			rb.gravityScale = 0;
			rb.isKinematic = true;
			completed = true;
			Stick();
			StartCoroutine(drink());
		}
	}

	void OnTriggerEnter2D(Collider2D coll) {
		if (coll.gameObject.CompareTag("Fernet")) {
			transform.parent = fernet.transform;
			rb.velocity = Vector2.zero;
			rb.gravityScale = 0;
			rb.isKinematic = true;
			completed = true;
			Stick();
		}
	}

	void OnCollisionExit2D (Collision2D coll) {
		canJump = false;
	}

	float moveHorizontallyTo (Vector2 point, ForceMode2D forceMode, float speed) {
		Vector2 offset = (point - new Vector2 (transform.position.x, transform.position.y)) * velocity / 3 * speed;
		offset.y = 0;

		if (Mathf.Abs (offset.x) < 5)
			offset.x = 0;
		else if (Mathf.Abs (offset.x) > topVelocity * 10 * speed)
			offset.x = topVelocity * speed * (offset.x > 0 ? 1 : -1);

		float magnitude = offset.magnitude;

		offset.x -= rb.velocity.x;

		rb.AddForce (offset, forceMode);
		animator.SetBool("seMueve", true);
		//Nota, Esto nomas se ejecuta cuando el enemy NO es fantasma
		if (rb.velocity.x > 0)
			spRenderer.flipX = true;
		else
			spRenderer.flipX = false;

		return magnitude;
	}

	void moveTo (Vector2 point) {
		Vector2 offset = (point - new Vector2 (transform.position.x, transform.position.y)) * ghostVelocity;

		float AbsX = Mathf.Abs(offset.x);
		float AbsY = Mathf.Abs(offset.y);
		float top = ghostVelocity * 1.5f;

		if (AbsX < 0.5 && AbsY > 0.5)
			offset.x = 0;
		else if (AbsX > 0.5 && AbsY < 0.5)
			offset.y = 0;

		if (AbsX > ghostVelocity * 1.5 || AbsY > top)
		if (AbsX > AbsY) {
			offset.y = top / offset.x * offset.y * offset.x > 0 ? 1 : -1;
			offset.x = offset.x > 0 ? top : -top;
		} else if (AbsY > AbsX) {
			offset.x = top / offset.y * offset.x * offset.y > 0 ? 1 : -1;
			offset.y = offset.y > 0 ? top : -top;
		} else {
			offset.x = offset.x > 0 ? top : -top;
			offset.y = offset.y > 0 ? top : -top;
		}

		offset -= rb.velocity;

		rb.AddForce (offset);
	}

	bool isLess (Vector2 comparator, Vector2 Comparated) {
		return comparator.sqrMagnitude < comparator.sqrMagnitude;
	}
}
