﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

public class GameManager : MonoBehaviour {

	GameObject musicPlayer;

	public List<GameObject> enemySpawns;
	public List<GameObject> plataforms;
	public List<GameObject> fernetPivots;
	public GameObject fernet;

	public GameObject enemyPrefab;
	public GameObject fernetLiquid;

	public GameObject player1;
	public GameObject player2;
	public Animator animatorF;

	public Text score1;
	public Text score2;
	public Text powers1;
	public Text powers2;
	public Text handicap1;
	public Text handicap2;

	public Text announcer;
	public Text fernetHP;

	public int wave = 0;
	public int cooldown = 5;

	JugadorScript controller1;
	JugadorScript controller2;

	FernetController fernetController;

	List<EnemyController> activeEnemies = new List<EnemyController>();

	float liquidStartY;
	float liquidScaleY;

	//Waves
	int enemyCount = 0;
	int currentEnemies = 0;
	int waveEnemies = 0;
	int capMonsters = 0;
	int fernetDamage = 0;

	float spawnCooldown = 0;
	float currentSpawnCooldown = 0;

	int battleTime = 0;

	enum GamePhase { Waves, Battle };
	GamePhase phase = GamePhase.Waves;
	public AudioClip musicaInGame;

	bool onWave = false;

	void Start() {
		musicPlayer = GameObject.FindGameObjectWithTag ("music");
		AudioSource tmpAudioS = musicPlayer.GetComponent<AudioSource> ();
		tmpAudioS.Stop ();
		tmpAudioS.clip = musicaInGame;
		tmpAudioS.Play ();
		controller1 = player1.GetComponent<JugadorScript>();
		controller2 = player2.GetComponent<JugadorScript>();

		liquidScaleY = fernetLiquid.transform.localScale.y;
		liquidStartY = fernetLiquid.transform.localPosition.y;

		fernetController = fernet.GetComponent<FernetController>();

		StartCoroutine(clock());

		fernetController.Refill();

		fernetHP.text = "Fernet: " + fernetController.getHP() + " (" + ((int) (10000f/fernetController.maxHP*fernetController.getHP()))/100f + "%)";
	}

	IEnumerator clock() {
		yield return new WaitForSecondsRealtime(1f);

		if (phase == GamePhase.Waves) {
			if (cooldown > 0) {
				announcer.text = "¡La oleada " + (wave + 1) + " comenzará en " + cooldown + " segundos!";
				cooldown--;
			} else if (!onWave) {
				wave++;
				startWave();
			} else {
				tickWave();
			}
		} else if (phase == GamePhase.Battle) {
			if (cooldown > 0) {				
				announcer.text = "¡Batalla comenzando en " + cooldown + "!";
				cooldown--;
			} else if (!onWave) {
				startBattle();
				onWave = true;
			} else {
				tickBattle();
			}
		}

		controller1.tickCooldown();
		controller2.tickCooldown();

		StartCoroutine(clock());
	}

	public void updateWaveUses(int id, int waveUses) {
		getPowersByID(id).text = "Powers: " + waveUses;
	}

	void startBattle() {
		controller1.stuned = false;
		controller2.stuned = false;

		announcer.text = "";
		if (controller1.score == 0 || controller2.score == 0)
			if (controller1.score == 0 && controller2.score == 0)
				controller1.cooldownMultiplier = controller2.cooldownMultiplier = 1;
			else if (controller1.score == 0 && controller2.score != 0) {
			controller1.cooldownMultiplier = 1;
			controller2.cooldownMultiplier = 0.7f;
			} else {
			controller2.cooldownMultiplier = 1;
			controller1.cooldownMultiplier = 0.7f;
			}
		else if (controller1.score > controller2.score) {
			controller1.cooldownMultiplier = controller1.score / controller2.score * 1.5f;
			controller2.cooldownMultiplier = controller2.score / controller1.score / 1.5f;
		} else {
			controller1.cooldownMultiplier = controller1.score / controller2.score / 1.5f;
			controller2.cooldownMultiplier = controller2.score / controller1.score * 1.5f;
		}

		controller1.waveUses = 3;
		controller2.waveUses = 3;

		updateWaveUses(1, 3);
		updateWaveUses(2, 3);

		if (controller1.handicap != JugadorScript.Handicap.NoThrowback) {
			controller1.handicap = JugadorScript.Handicap.None;
			handicap1.text = "";
		}

		if (controller2.handicap != JugadorScript.Handicap.NoThrowback) {
			controller2.handicap = JugadorScript.Handicap.None;
			handicap2.text = "";
		}

		score1.text = "";
		score2.text = "";
	}

	void startWave() {
		enemyCount = 0;
		currentEnemies = 0;
		announcer.text = "";
		waveEnemies = (int) (5f + wave * wave / 2f);
		capMonsters = 3 + wave;
		spawnCooldown = 3f / (wave * wave / 2f) * wave;
		currentSpawnCooldown = 0;
		fernetDamage = 10 + wave * wave;
		if (fernetDamage > 250)
			fernetDamage = 250;

		score1.text = "Puntuación: " + controller1.score;
		score2.text = "Puntuación: " + controller2.score;

		updateWaveUses(1, controller1.waveUses);
		updateWaveUses(2, controller2.waveUses);

		onWave = true;
	}

	void tickWave() {
		if (capMonsters > currentEnemies && waveEnemies > 0) {
			int toSpawn = 0;

			if (currentSpawnCooldown == 0) {
				toSpawn = 1;
				currentSpawnCooldown = spawnCooldown;
			} else if (currentSpawnCooldown < 1 && currentSpawnCooldown > 0) {
				toSpawn = (int) (1f / currentSpawnCooldown);
				currentSpawnCooldown = spawnCooldown;
			} else
				currentSpawnCooldown --;

			while(toSpawn > 0 && capMonsters > currentEnemies && waveEnemies > 0) {
				spawnEnemy();
				toSpawn--;
			}
		}
	}

	void tickBattle() {
		battleTime++;

		if (battleTime % 8 == 0) {
			controller1.addWaveUse();
			controller2.addWaveUse();
		}

	}

	void endWave() {
		cooldown = 10;

		onWave = false;

		foreach (EnemyController controller in activeEnemies)
			controller.Destroy();
		
		activeEnemies.Clear();
	}

	void finalizeWavesPhase() {
		endWave();

		controller1.armado = false;
		controller2.armado = false;

		cooldown = 5;

		controller1.stuned = true;
		controller2.stuned = true;

		fernetController.Refill();
		fernetLiquid.transform.localPosition = new Vector3(0, liquidStartY, 0);

		animatorF.SetTrigger("fadeOut");

		fernetHP.text = "";
		announcer.text = "";

		controller1.respawn();
		controller2.respawn();

		phase = GamePhase.Battle;
	}

	public void PlayerHitEvent(GameObject enemy, int ID) {
		JugadorScript controller = getControllerByID(ID);
		int damage = controller.GetDamage();
		EnemyController enemyController = enemy.GetComponent<EnemyController>();
		bool death = enemyController.Damage(damage, ID);

		if (death) {
			killEnemy(controller);
			activeEnemies.Remove(enemyController);
		} else if (!enemyController.ghostMode && !enemyController.completed) {
			Vector3 origin = controller.transform.position;
			enemyController.throwBack(origin, 2f);
		}
	}

	void killEnemy(JugadorScript controller) {
		controller.score += 1;

		if (controller.score % 10 == 0)
			controller.addWaveUse();

		getTextByID(controller.numero).text = "Puntuación: " + controller.score;

		currentEnemies--;
		if (waveEnemies < 1 && currentEnemies < 1)
			endWave();
	}

	public void PlayerLavaFallEvent(int ID) {
		if (phase == GamePhase.Battle) {
			getControllerByID(ID).generateHandicap();
			getHandicapByID(ID).text = "Handicap: " + getControllerByID(ID).handicap.ToString();

			int otherID = ID == 1 ? 2 : 1;

			getControllerByID(otherID).handicap = JugadorScript.Handicap.None;
			getHandicapByID(otherID).text = "";

			reLoop();
		} else
			getControllerByID(ID).respawn();
	}

	void reLoop() {
		wave = 0;

		controller1.respawn();
		controller2.respawn();

		controller1.score = 0;
		controller2.score = 0;

		controller1.waveUses = 2;
		controller2.waveUses = 2;

		updateWaveUses(1, 2);
		updateWaveUses(2, 2);

		controller1.armado = true;
		controller2.armado = true;

		battleTime = 0;

		cooldown = 5;

		animatorF.SetTrigger("fadeIn");

		fernetHP.text = "Fernet: " + fernetController.getHP() + " (" + ((int) (10000f/fernetController.maxHP*fernetController.getHP()))/100f + "%)";
		score1.text = "Puntuación: " + controller1.score;
		score2.text = "Puntuación: " + controller2.score;

		onWave = false;

		phase = GamePhase.Waves;
	}

	public void EnemyLavaFallEvent(GameObject enemy) {
		EnemyController enemyController = enemy.GetComponent<EnemyController>();
		if (enemyController.lastHitter != 0)
			killEnemy(getControllerByID(enemyController.lastHitter));
		activeEnemies.Remove(enemyController);
	}

	public void drinkFernet() {
		bool empty = fernetController.Damage(fernetDamage);

		if (empty) {
			fernetHP.text = "Fernet: 0 (0%)";
			finalizeWavesPhase();
		} else {
			float percentage = ((int) (10000f/fernetController.maxHP*fernetController.getHP()))/100f;

			fernetLiquid.transform.localPosition = new Vector3(0, liquidStartY - (liquidScaleY - liquidScaleY / 100 * percentage), 0);

			fernetHP.text = "Fernet: " + fernetController.getHP() + " (" + percentage + "%)";
		}
	}

	JugadorScript getControllerByID(int id) {
		return id == 1 ? controller1 : controller2;
	}

	Text getTextByID(int id) {
		return id == 1 ? score1 : score2;
	}

	Text getPowersByID(int id) {
		return id == 1 ? powers1 : powers2;
	}

	Text getHandicapByID(int id) {
		return id == 1 ? handicap1 : handicap2;
	}

	void spawnEnemy() {
		Vector3 spawnPosition = enemySpawns[Random.Range(0, enemySpawns.Count)].transform.position;

		GameObject casper = Instantiate(enemyPrefab, spawnPosition, Quaternion.identity) as GameObject;
		EnemyController controller = casper.GetComponent<EnemyController>();
		controller.plataforms = plataforms;
		controller.fernets = fernetPivots;
		controller.manager = this;

		controller.chooseFernet();
		controller.calcTargetPlataform();

		activeEnemies.Add(controller);

		enemyCount++;
		currentEnemies++;
		waveEnemies--;
	}
}
