﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class JugadorScript : MonoBehaviour {
	public int numero = 2, daño = 10, score = 0, waveUses = 2;
	public float velocidad, fuerzaOnda, fuerzaSalto, fuerzaPuño, cooldownMultiplier = 1;
	public AudioClip[] espadas, golpes;
	public AudioClip ondaSound;
	public GameManager manager;
	public GameObject ondaO;

	public enum Handicap { InvertAxis, Weakness, ExtraCooldown, NoThrowback, None };

	public Handicap handicap = Handicap.None;

	Animator animator;
	Rigidbody2D rigid;
	AudioSource audioS;

	string stringEjes;

	int cooldown = 0;

	KeyCode punchStringKey, powerStringKey, jumpStringKey;

	public bool armado = false;
	public bool stuned = false;
	bool onWave = false, hitPlayer = false;
	Vector3 startPos;

	public void generateHandicap() {
		float val = Random.Range(0, 5);
		if (val < 1)
			handicap = Handicap.InvertAxis;
		else if (val < 2)
			handicap = Handicap.Weakness;
		else if (val < 3)
			handicap = Handicap.ExtraCooldown;
		else
			handicap = Handicap.NoThrowback;
	}

	void Start () {
		animator = GetComponent<Animator> ();
		rigid = GetComponent<Rigidbody2D> ();
		audioS = GetComponent<AudioSource> ();
		startPos = transform.position;

		if (numero == 1) {
			punchStringKey = KeyCode.Keypad0;
			stringEjes = "Horizontal";
			powerStringKey = KeyCode.KeypadEnter;
			jumpStringKey = KeyCode.UpArrow;
		} else {
			punchStringKey = KeyCode.V;
			stringEjes = "Horizontal2";
			powerStringKey = KeyCode.B;
			jumpStringKey = KeyCode.W;
		}
	}

	IEnumerator jumpPatch() {
		yield return new WaitForSeconds(1f);
	}

	public void addWaveUse() {
		if (waveUses < 4) {
			waveUses++;
			manager.updateWaveUses(numero, waveUses);
		}
	}

	public void tickCooldown() {
		if (cooldown > 0)
			cooldown--;
		if (hitPlayer && cooldown < 1)
			hitPlayer = false;
	}

	public void respawn() {
		rigid.isKinematic = true;
		transform.position = startPos;
		rigid.isKinematic = false;
	}

	public int GetDamage() {
		if (onWave)
			return daño * (handicap == Handicap.Weakness ? 3 : 6);
		return daño / (handicap == Handicap.Weakness ? 2 : 1);
	}

	public void Atacar(){
		if (armado) {
			animator.SetTrigger ("blandirEspada");
			audioS.PlayOneShot (espadas[Random.Range(0, espadas.Length)]);
		} else{
			animator.SetTrigger ("pegar");
			audioS.PlayOneShot (golpes[Random.Range(0, golpes.Length)]);
		}
	}

	void Onda(){
		ondaO.SetActive(true);
		animator.SetTrigger ("onda");
		onWave = true;
		cooldown = (int) (handicap == Handicap.ExtraCooldown ? 5 : 3 * cooldownMultiplier);
		audioS.PlayOneShot(ondaSound);
		waveUses--;

		manager.updateWaveUses(numero, waveUses);
	}

	public void EndWave() {
		onWave = false;
	}

	void FixedUpdate(){
		if (!stuned) {
			MoverseYGirar ();
		}
		CheckJump ();
	}
	void Update () {		
		CheckForAtackKey ();
		CheckWalk ();
	}

	void CheckWalk(){
		if (rigid.velocity.x != 0)
			animator.SetBool ("moviendose", true);
		else {
			animator.SetBool ("moviendose", false);
		}
	}

	void MoverseYGirar(){
		Vector2 eje = new Vector2 ();
		eje.x = Input.GetAxis (stringEjes) * (handicap == Handicap.InvertAxis ? -1 : 1);
		if (rigid.velocity.x > 0) {
			transform.localScale = new Vector3 (1, 1, 1);
		} else if (rigid.velocity.x < 0) {
			transform.localScale = new Vector3 (-1, 1, 1);
		}
		rigid.AddForce(eje * velocidad, ForceMode2D.Force);

	}

	public void GetStuned(float time){
		StartCoroutine (Stunt(time));
	}

	void OnTriggerEnter2D(Collider2D other){
		if ((other.CompareTag ("Player") && !hitPlayer)|| other.CompareTag("bicho") && onWave) {
			if (handicap != Handicap.NoThrowback)	
				other.attachedRigidbody.AddForce ((other.transform.position - transform.position) * fuerzaOnda, ForceMode2D.Impulse);
			if (other.CompareTag("Player")) {
				other.GetComponent<JugadorScript> ().GetStuned(2f);
				hitPlayer = true;
			}
		} else if (other.CompareTag ("Player")) {
			other.attachedRigidbody.AddForce ((other.transform.position - transform.position) * fuerzaPuño, ForceMode2D.Impulse);
			if (!stuned)
				other.GetComponent<JugadorScript> ().GetStuned(.7f);
		}

		if (other.CompareTag("bicho"))
			manager.PlayerHitEvent(other.gameObject, numero);

	}

	IEnumerator Stunt(float time){
		stuned = true;
		yield return new WaitForSecondsRealtime (time);
		stuned = false;
	}

	void CheckForAtackKey(){
		if (!stuned && Input.GetKeyDown (punchStringKey)) {
			Atacar ();
		} else if (!stuned && !onWave && cooldown < 1 && waveUses > 0 && Input.GetKeyDown (powerStringKey)) {
			Onda ();
		}
	}

	public GameObject rayCastFloor;
	void CheckJump(){
		if (!stuned) {
		RaycastHit2D hit = Physics2D.Raycast (rayCastFloor.transform.position, Vector2.zero, 1f);
		if (hit.transform == null)
			return;
		if (Input.GetKeyDown(jumpStringKey)) {
			rigid.AddForce (new Vector2(0, fuerzaSalto), ForceMode2D.Impulse);
		}
	}
	}
}
